@@ -11,20 +11,14 @@
 #' @param wt_type Type of weight to use in the meta-analysis: native options are "sample_size", and "inv_var" (inverse error variance).
 #' Supported options borrowed from metafor are "DL", "HE", "HS", "SJ", "ML", "REML", "EB", and "PM"
 #' (see metafor documentation for details about the metafor methods).
-#' @param conf_level Confidence level to define the width of the confidence interval (default = .95).
-#' @param cred_level Credibility level to define the width of the credibility interval (default = .80).
-#' @param conf_method Distribution to be used to compute the width of confidence intervals. Available options are "t" for \emph{t} distribution or "norm" for normal distribution.
-#' @param cred_method Distribution to be used to compute the width of credibility intervals. Available options are "t" for \emph{t} distribution or "norm" for normal distribution.
-#' @param var_unbiased Logical scalar determining whether variances should be unbiased (\code{TRUE}) or maximum-likelihood (\code{FALSE}).
 #' @param moderators Matrix of moderator variables to be used in the meta-analysis (can be a vector in the case of one moderator).
 #' @param cat_moderators Logical scalar or vector identifying whether variables in the \code{moderators} argument are categorical variables (\code{TRUE}) or continuous variables (\code{FALSE}).
 #' @param moderator_type Type of moderator analysis ("none", "simple", or "hierarchical").
-#' @param hs_override When TRUE, this will override settings for \code{wt_type} (will set to "sample_size"),
-#' \code{conf_method} (will set to "norm"), \code{cred_method} (will set to "norm"), and \code{var_unbiased} (will set to \code{FALSE}).
 #' @param data Data frame containing columns whose names may be provided as arguments to vector arguments and/or moderators.
+#' @param control Output from the \code{control_psychmeta()} function or a list of arguments controlled by the \code{control_psychmeta()} function. Ellipsis arguments will be screened for internal inclusion in \code{control}.
 #' @param ... Further arguments to be passed to functions called within the meta-analysis.
 #'
-#' @return A list object of the classes \code{psychmeta}, \code{ma_generic}, and \code{ma_bb}.
+#' @return A nested tabular object of the class "ma_psychmeta".
 #'
 #' @export
 #'
@@ -34,13 +28,31 @@
 #' es <- c(.3, .5, .8)
 #' n <- c(100, 200, 150)
 #' var_e <- 1 / n
-#' ma_generic(es = es, n = n, var_e = var_e)
-ma_generic <- function(es, n, var_e, sample_id = NULL, citekey = NULL, wt_type = "sample_size",
-                         conf_level = .95, cred_level = .8, conf_method = "t", cred_method = "t", var_unbiased = TRUE,
-                         moderators = NULL, cat_moderators = TRUE, moderator_type = "simple", hs_override = FALSE, data = NULL, ...){
-     warn_obj1 <- record_warnings()
+#' ma_obj <- ma_generic(es = es, n = n, var_e = var_e)
+#' ma_obj
+#' summary(ma_obj)
+ma_generic <- function(es, n, var_e, sample_id = NULL, citekey = NULL, 
+                       wt_type = c("sample_size", "inv_var", 
+                                   "DL", "HE", "HS", "SJ", "ML", "REML", "EB", "PM"),
+                       moderators = NULL, cat_moderators = TRUE,
+                       moderator_type = c("simple", "hierarchical", "none"),
+                       data = NULL, control = control_psychmeta(), ...){
      call <- match.call()
+     warn_obj1 <- record_warnings()
 
+     wt_type <- match.arg(wt_type, choices = c("sample_size", "inv_var", 
+                                               "DL", "HE", "HS", "SJ", "ML", "REML", "EB", "PM"))
+     moderator_type <- match.arg(moderator_type, choices = c("simple", "hierarchical", "none"))
+     
+     control <- control_psychmeta(.psychmeta_ellipse_args = list(...),
+                                  .control_psychmeta_arg = control)
+     conf_level <- control$conf_level
+     cred_level <- control$cred_level
+     conf_method <- control$conf_method
+     cred_method <- control$cred_method
+     var_unbiased <- control$var_unbiased
+     hs_override <- control$hs_override
+     
      if(hs_override){
           wt_type <- "sample_size"
           conf_method <- cred_method <- "norm"
@@ -109,8 +121,10 @@ ma_generic <- function(es, n, var_e, sample_id = NULL, citekey = NULL, wt_type =
 
      additional_args <- list(...)
 
-     inputs <- list(wt_type = wt_type, conf_level = conf_level, cred_level = cred_level, conf_method = conf_method, cred_method = cred_method,
-                    var_unbiased = var_unbiased, cat_moderators = cat_moderators, moderator_type = moderator_type, data = data)
+     inputs <- list(wt_type = wt_type, 
+                    conf_level = conf_level, cred_level = cred_level, 
+                    conf_method = conf_method, cred_method = cred_method,
+                    var_unbiased = var_unbiased)
 
      es_data <- data.frame(es = es, n = n, var_e = var_e)
      if(is.null(sample_id)) sample_id <- paste0("Sample #", 1:nrow(es_data))
@@ -124,13 +138,17 @@ ma_generic <- function(es, n, var_e, sample_id = NULL, citekey = NULL, wt_type =
                                           conf_method = conf_method, cred_method = cred_method, var_unbiased = var_unbiased, wt_type = wt_type),
                        presorted_data = additional_args$presorted_data, analysis_id_variables = additional_args$analysis_id_variables,
                        moderator_levels = moderator_levels, moderator_names = moderator_names)
-     out$barebones <- append(list(call = call, inputs = inputs), out$barebones)
-     out <- append(list(call_history = list(call)), out)
-
-     out$barebones$messages <- list(warnings = clean_warning(warn_obj1 = warn_obj1, warn_obj2 = record_warnings()),
-                                    fyi = record_fyis(neg_var_res = sum(out$barebones$meta_table$var_res < 0, na.rm = TRUE)))
-
-     class(out) <- c("psychmeta", "ma_generic", "ma_bb")
+     
+     out <- bind_cols(analysis_id = 1:nrow(out), out)
+     attributes(out) <- append(attributes(out), list(call_history = list(call), 
+                                                     inputs = inputs, 
+                                                     ma_methods = "bb",
+                                                     ma_metric = "generic", 
+                                                     warnings = clean_warning(warn_obj1 = warn_obj1, warn_obj2 = record_warnings()),
+                                                     fyi = record_fyis(neg_var_res = sum(unlist(map(out$meta_tables, function(x) x$barebones$var_res < 0)), na.rm = TRUE)))) 
+     out <- namelists.ma_psychmeta(ma_obj = out)
+     
+     class(out) <- c("ma_psychmeta", class(out))
      return(out)
 }
 
@@ -209,9 +227,6 @@ ma_generic <- function(es, n, var_e, sample_id = NULL, citekey = NULL, wt_type =
      if(k == 1){
           var_es <- sd_es <- NA
           var_res <- sd_res <- NA
-          # se_es <- sd_e
-          # ci <- confidence(mean = mean_es, sd = sd_e, k = 1, conf_level = conf_level, conf_method = conf_method)
-
           se_es <- NA
           ci <- cbind(NA, NA)
           colnames(ci) <- paste("CI", c("LL", "UL"), round(conf_level * 100), sep = "_")
@@ -223,7 +238,7 @@ ma_generic <- function(es, n, var_e, sample_id = NULL, citekey = NULL, wt_type =
      ci <- setNames(c(ci), colnames(ci))
      cv <- setNames(c(cv), colnames(cv))
 
-     list(barebones = list(meta = data.frame(t(c(k = k,
+     list(meta = list(barebones = data.frame(t(c(k = k,
                                                  N = N,
                                                  mean_es = mean_es,
                                                  var_es = var_es,
@@ -233,8 +248,9 @@ ma_generic <- function(es, n, var_e, sample_id = NULL, citekey = NULL, wt_type =
                                                  se_es = se_es,
                                                  sd_e = sd_e,
                                                  sd_res = sd_res,
-                                                 ci, cv))),
-                           data = escalc_obj))
+                                                 ci, cv)))),
+          escalc = list(barebones = escalc_obj))
+
 }
 
 
@@ -250,7 +266,7 @@ ma_generic <- function(es, n, var_e, sample_id = NULL, citekey = NULL, wt_type =
 .ma_generic_boot <- function(data, i, ma_arg_list){
      data <- data[i,]
      out <- .ma_generic(data = data, run_lean = TRUE, ma_arg_list = ma_arg_list)
-     unlist(out$barebones$meta)
+     unlist(out$meta$barebones)
 }
 
 
